/*
    Protected Region Area
        あかつきみさき (みくちぃP)

    このスクリプトについて
        選択したレイヤー/キーフレーム範囲から,レスポンシブデザイン-時間の保護された領域を作成するスクリプト.
        レイヤーを選択している時
            選択レイヤーの最も小さいインポイント時間をマーカー時間にし,最も大きいアウトポイント時間をデュレーションに設定します.
        キーフレームを選択している時
            選択キーフレームの最も小さい時間をマーカー時間にし,最も大きい時間をデュレーションに設定します.

    使用方法
        ファイル→スクリプト→スクリプトファイルを実行.

    動作環境
        Adobe After Effects CC 2019以上

    バージョン情報
        2018/10/16 Ver 1.0.0 Release
*/

/// <reference path="C:/Users/RUI/OneDrive/lib/aftereffects.d.ts/ae.d.ts"/>

(function () {
    var isCompActive = function (comp) {
        if (!(comp && comp instanceof CompItem)) {
            return false;
        }
        else {
            return true;
        }
    };

    var isLayerSelected = function (layers) {
        if (layers.length === 0) {
            return false;
        }
        else {
            return true;
        }
    };

    if (parseFloat(app.version) < 16) {
        return 0;
    }

    var actComp = app.project.activeItem;
    if (!isCompActive(actComp)) {
        return 0;
    }

    app.beginUndoGroup("Protected Region Area");

    var min = Number.MAX_VALUE;
    var max = Number.MIN_VALUE;

    var selLayers = actComp.selectedLayers;
    var compMarkerText = "";
    if (!isLayerSelected(selLayers)) {
        min = actComp.time;
        max = 1;
    } else {
        compMarkerText = selLayers[selLayers.length - 1].name;
    }

    if (compMarkerText == null || compMarkerText == "") {
        compMarkerText = "";
    }

    for (var i = 0; i < selLayers.length; i++) {
        if (min > selLayers[i].inPoint) {
            min = selLayers[i].inPoint;
        }

        if (max < selLayers[i].outPoint) {
            max = selLayers[i].outPoint;
        }
    }

    var selProperty;
    var curKeyTime = 0;
    var isSelectedKerys = false;

    for (var i = 0; i < selLayers.length; i++) {
        for (var j = 0; j < selLayers[i].selectedProperties.length; j++) {
            selProperty = selLayers[i].selectedProperties[j];

            if (!(selProperty instanceof Property)) {
                continue;
            }

            for (var k = 0; k < selProperty.selectedKeys.length; k++) {
                if (!isSelectedKerys) {
                    min = Number.MAX_VALUE;
                    max = Number.MIN_VALUE;
                    isSelectedKerys = true;
                }

                curKeyTime = selProperty.keyTime(selProperty.selectedKeys[k]);

                if (min > curKeyTime) {
                    min = curKeyTime;
                }

                if (max < curKeyTime) {
                    max = curKeyTime;
                }
            }

            compMarkerText = selLayers[i].selectedProperties[selLayers[i].selectedProperties.length - 1].name;
        }
    }

    var markerObj = new MarkerValue(compMarkerText);
    if (max - min > 0) {
        markerObj.duration = max - min;
        markerObj.protectedRegion = true;
    }

    if (isLayerSelected(selLayers)) {
        markerObj.label = selLayers[selLayers.length - 1].label;
    }

    actComp.markerProperty.setValueAtTime(min, markerObj);
    app.endUndoGroup();
    return 0;
}).call(this);
